#!/bin/sh
#--------------------------------------------------------------------------
# Licensed Materials - Property of IBM
# 5724-C04
#
# (C) Copyright IBM Corporation 2005. All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication 
# or disclosure restricted by GSA ADP Schedule Contract 
# with IBM Corp.
#--------------------------------------------------------------------------
#
# Stops all of the processes that make up the UNIX Agent.
#
#--------------------------------------------------------------------------

# Gets the PIDs of the children of a process.
getChildPids()
{
    ppid=$1
    grep $ppid $TMP_FILE | awk '{if ($3 == '"$ppid"') print $2}'
}

# Gets the PIDs of the children of a process.
confirmPid()
{
    pid=$1
    ps -ef | awk '{if ($2 == '"$pid"') print $2}'
}

# Kills a process and all of its sub-processes.
#
# Because this function is called recursively, it only uses
# variables $1, $2, etc.  
doKill()
{
    for child in `getChildPids $1`
    do
        doKill $child
    done
    for pid_alive in `confirmPid $1`
    do
        kill -9 $1
        echo "Killed PID" $1".  Return code: " $?
    done
}

# If the CANDLEHOME environment variable is not set, we can't proceed.
if [ -z "$CANDLEHOME" ]
then
    echo "The CANDLEHOME variable is not set."
    exit 1
fi

# Pipe the output of the ps command to a temporary file so that we
# can scan it for the UNIX Agent sub-processes.
TMP_FILE=/tmp/.kuxtmp.$$

rm -f $TMP_FILE

ps -ef | grep -v itmpatch > $TMP_FILE

# Data on running processes is saved by the CandleAgent command so that 
# it can stop the right process.  Get the UNIX Agent PID from this file.
runFile="$CANDLEHOME/config/.ConfigData/RunInfo"

kuxPid=`cat $runFile | awk -F\| '{if ($2 == "ux") print $3}'`
if [ -z "$kuxPid" ]
then
    # No $CANDLEHOME/kuxagent running
    exit 0
fi

echo "Killing kuxagent" $kuxPid
doKill $kuxPid

# This sleep is needed on HP-UX to allow the kuxagent file to be
# replaced.
sleep 3

# rm -f $TMP_FILE

echo "killkux returning " $?

